"use strict";

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }

/* eslint-disable no-console */
jest.setTimeout(25000);

const fs = require("./fs");

const path = require("path");

const babel = require("@babel/core");

const parseArgs = require("./argParser");
/**
 * Jest changes __dirname to relative path and the require of relative path
 * that doesn't start with "." will be a module require -
 *
 * require("./packages/babel-plugin...") vs require("packages/babel-plugin..");
 *
 * So we start the path with a "./"
 */


function pathJoin(...parts) {
  if (path.isAbsolute(parts[0])) {
    return path.join(...parts);
  }

  return "." + path.sep + path.join(...parts);
}

function testRunner(dir) {
  const pkgDir = pathJoin(dir, "../");
  const packageJson = JSON.parse(fs.readFileSync(pathJoin(pkgDir, "package.json")));
  const pkgName = packageJson.name;
  const fixturesDir = pathJoin(pkgDir, "__tests__/fixtures");
  const fixtures = fs.readdirSync(fixturesDir).filter(dir => fs.isDirectorySync(pathJoin(fixturesDir, dir)));
  const flags = parseArgs(process.argv);
  const updateFixtures = Boolean(flags["update-fixtures"]);
  describe(pkgName, () => {
    var _iterator = _createForOfIteratorHelper(fixtures),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        const fixture = _step.value;
        const actualFile = pathJoin(fixturesDir, fixture, "actual.js");
        const expectedFile = pathJoin(fixturesDir, fixture, "expected.js");
        const skipFile = pathJoin(fixturesDir, fixture, "skip");
        const optionsFile = pathJoin(fixturesDir, fixture, "options.json");
        const babelOptionsFile = pathJoin(fixturesDir, fixture, "babel.json");

        if (fs.isFileSync(skipFile)) {
          test.skip(fixture, () => {});
          continue;
        }

        test(fixture, /*#__PURE__*/_asyncToGenerator(function* () {
          const actual = yield fs.readFile(actualFile);
          let options = {};

          if (yield fs.isFile(optionsFile)) {
            options = JSON.parse(yield fs.readFile(optionsFile));
          }

          let babelOpts = {
            // set the default sourcetype to be script
            sourceType: "script"
          };

          if (yield fs.isFile(babelOptionsFile)) {
            Object.assign(babelOpts, JSON.parse(yield fs.readFile(babelOptionsFile)));
          }

          const currentPlugin = pathJoin(pkgDir, "src/index.js");

          if (Array.isArray(babelOpts.plugins)) {
            babelOpts.plugins = [[currentPlugin, options], ...babelOpts.plugins];
          } else {
            babelOpts.plugins = [[currentPlugin, options]];
          } // don't consider the project's babel.config.js


          babelOpts.configFile = false;
          const actualTransformed = babel.transformSync(actual, babelOpts).code;

          if (!(yield fs.isFile(expectedFile))) {
            yield fs.writeFile(expectedFile, actualTransformed);
            console.warn("Created fixture's expected file - " + expectedFile);
          } else if (updateFixtures) {
            const expected = yield fs.readFile(expectedFile);

            if (expected !== actualTransformed) {
              yield fs.writeFile(expectedFile, actualTransformed);
              console.warn("Updated fixture's expected file - " + expectedFile);
            }
          } else {
            const expected = yield fs.readFile(expectedFile);
            expect(actualTransformed).toBe(expected);
          }
        }));
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  });
}

module.exports = testRunner;